#!/usr/bin/env python3

import argparse
import os
import subprocess
import sys

import bee_common
import bee_config

# hiding traceback
sys.tracebacklimit=0

# argv[0] should be the command name
def exec_db_upgrade(argv, config_path):
    # storing name of test config file
    test_config_path ="config.test"

    # find and parse the configuration file first
    if(config_path == bee_config.DEFAULT_PATH or config_path == test_config_path):
        config = bee_config.Server(config_path)
    else:
        config = bee_config.envServer(bee_config.DEFAULT_PATH)

    # parse the arguments
    parser = argparse.ArgumentParser(description='BeeGFS Hive index database upgrade', add_help=False)
    # override help to not use -h
    parser.add_argument('--help',                                           action='help',               help='Show this help message and exit')
    parser.add_argument('-T',                   dest='upgrade_to_version',  action='store', type=str,    help='Hive index database upgrade to version',   required=False)
    parser.add_argument('-I',                   dest='index_path',          action='store', type=str,    help='Index directory path', default='')
    parser.add_argument('-b',   dest='backup_db',           action='store_true',    help='Backup database files while upgrading hive index database and do not remove after backup successful')
    parser.add_argument('-d',   dest='delete_backup_db',    action='store_true',    help='Delete Backup database files from each subdirectory recursively')
    parser.add_argument('-r',   dest='restore_backup_db',   action='store_true',    help='Restore Backup database files from each subdirectory recursively')
    parser.add_argument('-n',   dest='max_threads',         action='store', type=str,    help='provide number of threads',   required=False)


    args = parser.parse_args(argv[2:])

    query = []
    paths = []
    query_cmd = []

    # Get the fs_path and index_directory_path from config file.
    paths = ['']
    if(config_path == bee_config.DEFAULT_PATH or config_path == test_config_path):
        cpath = config.indexpaths()
    else:
        cpath = config_path

    fspath = [os.path.normpath(os.path.sep.join([cpath, path])) for path in paths]
    indexpaths = fspath[0].split(':')
    config_srcpath = indexpaths[0]

    if len(indexpaths) == 2:
        config_idxPath = indexpaths[1]
    else:
        config_idxPath = ""
    
    # Check if -I is not passed from cmdline and config file also do not have index directory path.
    if args.index_path == "" and config_idxPath == "":
        print("Index directory path not passed from cmdline and also from config file.")
        sys.exit()
    else:
        # If -I is not passed but config file has index directory path, use that path for index directory.
        if args.index_path == "":
            args.index_path = config_idxPath

        exec_path = "%s/bin/index/bee-db-upgrade" % config.executable()
        if args.delete_backup_db or args.restore_backup_db:
            # Query command for delete backup or restore db file
            # Ignore target version
            query_cmd = [exec_path]
        else:
            if args.upgrade_to_version == None:
            	print("error: the following arguments are required: -T")
            	print("Check \"man bee db-upgrade\"")
            	sys.exit()
            # Query command for upgrade or downgrade with backup db file
            query_cmd = [exec_path, '-T', args.upgrade_to_version]

        query_cmd += ['-i', args.index_path]
        # If -b is passed from cli then database file backup feature is enabled
        if args.backup_db:
            query_cmd += ['-b']
        if args.delete_backup_db:
            # Delete backup
            query_cmd += ['-D']
        if args.restore_backup_db:
            # Restore backup
            query_cmd += ['-r']
        if args.max_threads:
            # Max threads
            query_cmd += ['-n', args.max_threads]
        print(query_cmd)

    query = subprocess.Popen(query_cmd)

    query.communicate() # block until query finishes

    return query.returncode
