#!/usr/bin/env python3
# This file is part of GUFI, which is part of MarFS, which is released
# under the BSD license.
#
#
# Copyright (c) 2017, Los Alamos National Security (LANS), LLC
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation and/or
# other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors
# may be used to endorse or promote products derived from this software without
# specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#
# From Los Alamos National Security, LLC:
# LA-CC-15-039
#
# Copyright (c) 2017, Los Alamos National Security, LLC All rights reserved.
# Copyright 2017. Los Alamos National Security, LLC. This software was produced
# under U.S. Government contract DE-AC52-06NA25396 for Los Alamos National
# Laboratory (LANL), which is operated by Los Alamos National Security, LLC for
# the U.S. Department of Energy. The U.S. Government has rights to use,
# reproduce, and distribute this software.  NEITHER THE GOVERNMENT NOR LOS
# ALAMOS NATIONAL SECURITY, LLC MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR
# ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.  If software is
# modified to produce derivative works, such modified software should be
# clearly marked, so as not to confuse it with the version available from
# LANL.

# THIS SOFTWARE IS PROVIDED BY LOS ALAMOS NATIONAL SECURITY, LLC AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL LOS ALAMOS NATIONAL SECURITY, LLC OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
# OF SUCH DAMAGE.



import argparse
import os
import subprocess
import sys

import bee_common
import bee_config

# hiding traceback
sys.tracebacklimit=0

# argv[0] should be the command name
def exec_query_index(argv, config_path):
    # storing name of test config file
    test_config_path ="config.test"

    # find and parse the configuration file first
    if(config_path == bee_config.DEFAULT_PATH or config_path == test_config_path):
        config = bee_config.Server(config_path)
    else:
        config = bee_config.envServer(bee_config.DEFAULT_PATH)

    # parse the arguments
    parser = argparse.ArgumentParser(description='BeeGFS Hive index version of query-index', add_help=False)
    # override help to not use -h
    parser.add_argument('--help',                                           action='help',                                      help='show this help message and exit')
    parser.add_argument('-I',                   dest='db_path',             action='store',           type=str,                 help='Index directory path', default='')
    parser.add_argument('-s',                   dest='sql_query',           action='append',          type=str,                 help='Provide sql query')

    args = parser.parse_args(argv[2:])

    query = []
    query_cmd = []

    config_paths = config.indexpaths().split(':')
    fs_path = config_paths[0]

    if len(config_paths) == 2:
        index_path = config_paths[1]
    else:
        index_path = config_paths[0]

    path = ''.join(''.join(args.db_path))
    if(args.db_path):
        path = str(args.db_path)

    if len(path) == 0 or path[0] != "/":
        current_dir = os.getcwd()

        if len(path) == 0:
            path1 = current_dir

        else:
            path1 = "%s/%s" % (current_dir, path)

    else:
        path1 = path


    # check is path is part of index or FSpath
    if not path1.startswith(index_path) and not path1.startswith(fs_path):
        print("Invalid path. Please provide path, it should be index or FSpath")
        exit(1)

    # If the path starts with fspath, replace it with index
    if path1.startswith(fs_path):
        path1 = path1.replace(fs_path, index_path)

    fullpath = path1

    #if dir. path is not given then through error
    if fullpath == "" or fullpath == None:
        print("bee query-index: missing operand\nTry 'bee stat --help' for more information")
        exit(1)

    exec_path = "%s/bin/index/bee-query-index" % config.executable()
    query_cmd = [exec_path, fullpath]

    # If -s is passed from cli then append sql_query
    if args.sql_query:
        query_cmd += (args.sql_query)
    else:
        print("Please provide sql query")

    query = subprocess.Popen(query_cmd)

    query.communicate() # block until query finishes

    return query.returncode
